/**
 * PaletteBuilder 1.0
 * Copyright (c) 2010 Paul A. Jungwirth
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
// TODO: add a drop shadow to the main box?
(function($) {
	$.palettebuilder = {
	display: function(options) {
		function d2h(d) {
			var r = parseInt(d).toString(16);
			if (r.length == 0) return '00';
			else if (r.length == 1) return '0' + r;
			else return r;
		}
		function html_rgb(css_rgb) {
			if (css_rgb.match(/(\d+), *(\d+), *(\d+)/)) {
				return "#" + d2h(RegExp.$1) + d2h(RegExp.$2) + d2h(RegExp.$3);
			} else {
				return "#000000";
			}
		}
		function show_color(css_rgb, encoding) {
			if (encoding == 'html-rgb') {
				return html_rgb(css_rgb);
			} else if (encoding == 'css-rgb') {
				return css_rgb;
			} else {
				return css_rgb;
			}
		}
		// TODO: complain if there are no colors yet.
		var s = $.extend({
			divid: 'palettebuilder',
			color: 'black',
			bgcolor: 'white',
			left: '0px',
			top: '0px',
			z: '1000',
			color_encoding: 'html-rgb'	// TODO: convert farbtastic results back to rgb(x,y,z) when html-css is selected.
		}, options);
		var cws = $.palettebuilder.colorWells;
		var len = cws;
		var html = "<div id='" + s.divid + "' style='position:absolute; left:" + s.left + "; top:" + s.top + "; z-index:" + s.z + "; color: " + s.color + "; background-color: " + s.bgcolor + "; border: solid " + s.color + " 3px; padding:10px'>";
		html += "<div id='palettebuilder_color_wheel' style='float:left; padding:5px'>color wheel here</div>"
		html += "<div style='float:left'>";	// main right-side container
		for (var i in cws) {
			var o = cws[i];
			var selstr = o.selector_string();
			var c = o.color();
			// var c = '#ff0000';
			// alert(selstr + ": " + html_rgb(c));
			html += "<div style='padding:5px; clear:both' class='palettebuilder_colorrow'>";
			html += "<div style='border:solid " + s.color + " 1px; margin:0px 2px 4px 2px; float:left; width:40px; height:40px; background-color: " + c + "' class='palettebuilder_colorbox'>&nbsp;</div>";
			html += "<div style='float:left; margin-left:3px; max-width:300px'><div>" + selstr + "</div><div class='palettebuilder_color_encoding'>" + show_color(c, s.color_encoding) + "</div></div></div>";
		}

		html += "</div>";
		$('body').append(html);
		d = $('#' + s.divid);

		// each colorrow should store its colorWell object:
		$('.palettebuilder_colorrow', d).each(function(i) {
			this.palettebuilder_data = cws[i];
		});

		// when the colorwheel changes:
		//   identify the currently-selected colorrow
		//     (stored in $.palettebuilder.selected)
		//   set its colorbox to the new color
		//   set its selected DOM elements to the new color

		// listener for selecting a new color for the current colorWell:
		var farb = $.farbtastic('#palettebuilder_color_wheel', function(new_color) {
			var p = $.palettebuilder.selected;
			var o = p.palettebuilder_data;
			$('.palettebuilder_colorbox', p).css('background-color', new_color);
			$('.palettebuilder_color_encoding', p).html(new_color);
			o.newColor(new_color);
		});

		// listener for setting the current colorWell:
		$('.palettebuilder_colorrow', d).click(function(ev) {
			$('.palettebuilder_colorbox', d).each(function () {
				$(this).css('border', 'solid ' + s.color + ' 1px')
				.css('margin', '0px 2px 4px 2px');
			});
			var cb = $('.palettebuilder_colorbox', this)
			.css('border', 'solid ' + s.color + ' 3px')
			.css('margin', '0px 0px 0px 0px');
			$.palettebuilder.selected = this;
			farb.setColor(html_rgb(cb.css('background-color')));
		}).first().click();

		// only make the palette draggable if jquery-ui is present:
		if (d.draggable) d.draggable();

		// return something
	},
	addColor: function(fg_selectors, bg_selectors, border_selectors) {
		// Returns a jQuery object based on the given selectors.
		// Returns null if passed null or an empty list.
		function find_elements(sel) {
			if (sel == null) return null;
			if (typeof sel == 'string') {
				return $(sel);
			} else {
				var len = sel.length;
				if (len > 0) {
					var j = $(sel[0]);
					for (var i = 1; i < len; i++) {
						j = j.add(sel[i]);
					}
					return j;
				} else {
					return null;
				}
			}
		}
		var o = {};
		o.fgsel = (typeof fg_selectors == 'string') ? [fg_selectors] : fg_selectors;
		o.bgsel = (typeof bg_selectors == 'string') ? [bg_selectors] : bg_selectors;
		o.bsel = (typeof border_selectors == 'string') ? [border_selectors] : border_selectors;
		o.fg = find_elements(fg_selectors);
		o.bg = find_elements(bg_selectors);
		o.b = find_elements(border_selectors);

		// o.all = o.fg == null ? null : o.fg.clone();
		// if (o.bg != null) o.all = o.all == null ? o.bg : o.all.add(o.bg);
		// if (o.b != null) o.all = o.all == null ? o.b : a.all.add(o.b);
		
		o.selector_string = function() {
			var r = [];
			if (this.fgsel != null) r = r.concat(this.fgsel);
			if (this.bgsel != null) r = r.concat(this.bgsel);
			if (this.bsel != null)  r = r.concat(this.bsel);
			return r.join(', ');
		};
		o.color = function() {
			if (o.fg) return o.fg.css('color');
			else if (o.bg) return o.bg.css('background-color');
			else return o.b.css('border-color');
		}
		o.newColor = function(c) {
			if (o.fg) o.fg.css('color', c);
			if (o.bg) o.bg.css('background-color', c);
			if (o.b) o.b.css('border-color', c);
		}
		$.palettebuilder.colorWells.push(o);
	},
	colorWells: []
	};
})(jQuery);
